//  File:  SoupTest\SoupTest.java
//
//  A simple producer-consumer thread example using wait and notify

public class SoupTest {

    public static void main(String[] args) {
        System.out.println("Starting the SoupTest...");
        Soup s = new Soup();
        Producer p1 = new Producer(s);
        Consumer c1 = new Consumer(s);
        p1.start();
        c1.start();
        System.out.println("SoupTest main method completes");
        System.out.println("... but the feast continues ...");
    }

} // End class SoupTest


class Producer extends Thread {

    public Producer(Soup s) {
        soup = s;
    }

    public void run() {
        for (int i = 0; i < 10; i++) {
            char c = (char) ('A' + 26 * Math.random());
            soup.add(c);
            System.out.println("Added " + c + " to the soup (" + (i+1) + ")");
            try {
                sleep((int)(100.0 * Math.random()));
            } catch (InterruptedException e) { }
        }
    }

    private Soup soup;

} // End class Producer


class Consumer extends Thread {

    public Consumer(Soup s) {
        soup = s;
    }

    public void run() {
        for (int i = 0; i < 10; i++) {
            char c = soup.eat();
            System.out.println("Ate a letter: " + c + " (" + (i+1) + ")");
            try {
                sleep((int) (2000.0 * Math.random()));
            } catch (InterruptedException e) { }
        }
    }

    private Soup soup;

} // End class Consumer

class Soup {

    public synchronized char eat() {
        while (isEmpty) {
            try {
                wait();
            } catch (InterruptedException e) { }
        }
        char cToReturn = buffer[eatNext];
        eatNext = (eatNext + 1) % 6;
        if (eatNext == addNext)
            isEmpty = true;
        isFull = false;
        notify();
        return cToReturn;
    }

    public synchronized void add(char c) {
        while (isFull) {
            try {
                wait();
            } catch (InterruptedException e) { }
        }
        buffer[addNext] = c;
        addNext = (addNext + 1) % 6;
        if (addNext == eatNext)
            isFull = true;
        isEmpty = false;
        notify();
    }

    private char[] buffer   = new char[6];
    private int eatNext     = 0;
    private int addNext     = 0;
    private boolean isFull  = false;
    private boolean isEmpty = true;

} // End class Soup


/* ..... Output from the program .....
Starting the SoupTest...
SoupTest main method completes
... but the feast continues ...
Added H to the soup (1)
Ate a letter: H (1)
Added F to the soup (2)
Added T to the soup (3)
Added E to the soup (4)
Added J to the soup (5)
Added R to the soup (6)
Added V to the soup (7)
Ate a letter: F (2)
Added N to the soup (8)
Ate a letter: T (3)
Added L to the soup (9)
Ate a letter: E (4)
Added M to the soup (10)
Ate a letter: J (5)
Ate a letter: R (6)
Ate a letter: V (7)
Ate a letter: N (8)
Ate a letter: L (9)
Ate a letter: M (10)
..... End of Output ..... */
